<?php

/**
 * Base Class File for Authoroze.net APIs.
 * All AuthorizeNet APIs must extend from this Base Class
 * -
 * Sigup for test account
 * ----------------------
 * Sign up for a {@link: https://developer.authorize.net/testaccount/ Test Account} to obtain
 * an API Login ID and Transaction Key. These keys will authenticate requests to the payment gateway.
 * user: D********007
 * pass: N********123
 * acct: https://sandbox.authorize.net/UI/themes/sandbox/CustomerProfile/About.aspx
 * -
 * Download the Authorize.Net PHP SDK and include it in your project.
 * -----------------------------------------------------------------
 * The Authorize.Net PHP SDK gives you access to the full suite of APIs.
 * {@link https://developer.authorize.net/integration/fifteenminutes/#php_sdk Download the SDK}
 * You'll need to include the file "anet_php_sdk/AuthorizeNet.php" in your project so make sure you save
 * the SDK into a folder that your web server can access such as your var/www folder or your htdocs folder.
 * House: the downloaded files as root/thirdparty/anet_php_sdk
 * -
 * About AuthorizeNet
 * ==================
 * 1. Recurring Billing
 * 2. eCheck Processing
 * 3. API choices
 *    a. Simple Integration Method (SIM)
 *    b. Advanced Integration Method (AIM)
 * 4. Fraud Prevention/Fraud Detection Suite (FDS)
 *    - identify fraudulent transactions before your product ships
 *    a. amount filter: sets upper and lower value limits for "acceptable" sales
 *    b. velocity filter: limits the number of transactions that can be received per hour from a given customer
 *    c. IP filter and IP blocking functionality
 * 5. DDoS Attack prevention
 * -
 * Other Information
 * =================
 * Set up Fee: ?
 * Monthly Fee: ?
 * Transaction Fee: ?
 * Recurring Billing Fee: ?
 * FDS Fee: ?
 * eCheck Fee ?
 * SSL Fee ?
 * -
 * @author Deepak Pradhan
 * @version 0.01
 * @package AuthorizeNet.Base
 * @subpackage Base
 */
class authBase extends CApplicationComponent {

	// Setup the API Login Id, Transaction Key, Your MD5 Setting, and sandbox
	protected $api_lid = '4Amn87xxx';
	protected $txn_key = '99Z7y4xxx';
	protected $md5_stg = '4Amn87xxx';
	protected $ver_aim = '3.1';
	protected $retries = 2;
	protected $sandbox = true;
	// API Urls - Need not change till AuthorizeNet changes it
	protected $arb_api = array(
		'sandbox'=>'https://apitest.authorize.net/xml/v1/request.api',
		'production'=>'https://api.authorize.net/xml/v1/request.api'
	);
	protected $aim_api = array(
		'sandbox'=>'https://test.authorize.net/gateway/transact.dll',
		'production'=>'https://secure.authorize.net/gateway/transact.dll');
	protected $sim_api = array(
		'sandbox'=>'',
		'production'=>'');
	protected $dpm_api = array(
		'sandbox'=>'',
		'production'=>'');
	protected $api_url; // this url varies as to what API is used to extend this base

	public function init() {
		parent::init();
	}


	/**
	 * Set the appropiate API URL and the credentials for Authentication
	 * @param string $api API to use
	 */
	protected function setApi($api) {
		switch ($api) {
			case 'ARB':
				$this->api_url = ($this->sandbox) ? $this->arb_api['sandbox'] : $this->arb_api['production'];
				$this->params['merchantAuthentication']['name'] = $this->api_lid;
				$this->params['merchantAuthentication']['transactionKey'] = $this->txn_key;
				break;
			case 'AIM':
				$this->api_url = ($this->sandbox) ? $this->aim_api['sandbox'] : $this->aim_api['production'];
				$this->params['x_login'] = $this->api_lid;
				$this->params['x_tran_key'] = $this->txn_key;
				$this->params['x_version'] = $this->ver_aim;
				break;
			case 'SIM':
				$this->api_url = ($this->sandbox) ? $this->sim_api['sandbox'] : $this->sim_api['production'];
				break;
			case 'DPM':
				$this->api_url = ($this->sandbox) ? $this->dpm_api['sandbox'] : $this->dpm_api['production'];
				break;
			default:
				// trap error here
				return false;
				break;
		}
		return true;
	}




}