<?php

/**
 * All validation for ARB API goes here
 * Attribute defs taken from : http://www.phpclasses.org/browse/file/23553.html
 * @author Deepak Pradhan
 * @version 0.01
 * @package AuthorizeNet.Base
 * @subpackage ARB.Form
 * @todo refer to https://api.authorize.net/xml/v1/schema/AnetApiSchema.xsd and
 *       http://www.authorize.net/support/ARB_guide.pdf to make this form validation SOLID
 */
class AuthFormARB extends AuthForm {

	/**
	 * refId (optional): Merchant-assigned reference ID for the request. If included in the request, this
	 * value will be included in the response. This feature might be especially useful for multithreaded
	 * applications.
	 */
	public $refId;
	/**
	 * subscription: Merchant-assigned name for the subscription
	 */
	public $name;
	/**
	 * subscription/paymentSchedule/interval/length
	 * The measurment of time, in association with the Interval Unit, that is used to define the frequency
	 * of the billing occurences
	 */
	public $length;
	/**
	 * subscription/paymentSchedule/interval/unit
	 * The unit of time, in association with the Interval Length, between each billing occurence
	 */
	public $unit;
	/**
	 * subscription/paymentSchedule/startDate
	 * The date the subscription begins (also the date the initial billing occurs)
	 */
	public $startDate;
	/**
	 * subscription/paymentSchedule/totalOccurrences
	 * Number of billing occurrences or payments for the subscription, 9999 is max
	 */
	public $totalOccurrences = 9999;
	/**
	 * totalOccurrencesChanged
	 * Used to only in Update, to see if we have to send a new value
	 */
	public $totalOccurrencesChanged;
	/**
	 * subscription/paymentSchedule/trialOccurrences (optional)
	 * Number of billing occurrences or payments in the trial period
	 */
	public $trialOccurrences;
	/**
	 * subscription/amount
	 * The amount to be billed to the customer for each payment in the subscription
	 */
	public $amount;
	/**
	 * subscription/trialAmount (conditional)
	 * The amount to be charged for each payment during a trial period
	 */
	public $trialAmount;
	/**
	 * subscription/payment/creditCard/expirationDate
	 * The expiration date of the credit card used for the subscription, YYYY-MM
	 */
	public $expirationDate;
	public $expirationYear;
	public $expirationMonth;
	/**
	 * subscription/payment/bankAccount/accountType
	 * The type of bank account used for payment of the subscription,
	 * checking, businessChecking, savings
	 */
	public $accountType;
	/**
	 * subscription/payment/bankAccount/routingNumber
	 * The routing number of the customer's bank, 9 digits
	 */
	public $routingNumber;
	/**
	 * subscription/payment/bankAccount/accountNumber
	 * The bank account number used for payment of the subscription, 5 to 17 digits
	 */
	public $accountNumber;
	/**
	 * subscription/payment/bankAccount/nameOnAccount
	 * The full name of the individual associated with the bank account number
	 */
	public $nameOnAccount;
	/**
	 * subscription/payment/bankAccount/bankName
	 * The name of the bank associated with the bank account number
	 */
	public $bankName;
	/**
	 * subscription/payment/bankAccount/echeckType
	 * The type of electronic check transaction used for the subscription, PPD, WEB, CCD
	 */
	public $echeckType;
	/**
	 * subscription/order/invoiceNumber (optional)
	 * Merchant-assigned invoice number for the subscription
	 */
	public $invoiceNumber;
	/**
	 * /subscription/order/description (optional)
	 * Description of the subscription
	 */
	public $description;
	/**
	 * subscription/customer (optional)
	 * Contains information about the customer
	 */
	public $id;
	/**
	 * subscription/customer/email (optional)
	 * The customer's email address
	 */
	public $email;
	/**
	 * subscription/customer/phoneNumber (optional)
	 * The customer's phone number
	 */
	public $phoneNumber;
	/**
	 * subscription/customer/faxNumber (optional)
	 * The customer's fax number
	 */
	public $faxNumber;
	/**
	 * subscription/billTo/firstName
	 * The first name associated with the customer's billing address
	 */
	public $billFirstName;
	/**
	 * subscription/billTo/lastName
	 * The last name associated with the customer's billing address
	 */
	public $billLastName;
	/**
	 * /subscription/billTo/company (optional)
	 * The company associated with the customer's billing address
	 */
	public $billCompany;
	/**
	 * /subscription/billTo/address (optional)
	 * The customer's billing address
	 */
	public $billAddress;
	/**
	 * /subscription/billTo/city (optional)
	 * The city of the customer's billing address
	 */
	public $billCity;
	/**
	 * /subscription/billTo/state (optional)
	 * The state of the customer's billing address
	 */
	public $billState;
	/**
	 * subscription/billTo/zip (optional)
	 * The ZIP code of the customer's billing address
	 */
	public $billZip;
	/**
	 * subscription/billTo/country (optional)
	 * The country of the customer's billing address
	 */
	public $billCountry;
	/**
	 * subscription/shipTo/firstName
	 * The first name associated with the customer's shipping address
	 */
	public $shipFirstName;
	/**
	 * subscription/shipTo/lastName
	 * The last name associated with the customer's shipping address
	 */
	public $shipLastName;
	/**
	 * subscription/shipTo/company (optional)
	 * The company associated with the customer's shipping address
	 */
	public $shipCompany;
	/**
	 * subscription/shipTo/address (optional)
	 * The customer's shipping address
	 */
	public $shipAddress;
	/**
	 * subscription/shipTo/city (optional)
	 * The city of the customer's shipping address
	 */
	public $shipCity;
	/**
	 * subscription/shipTo/state (optional)
	 * The state of the customer's shipping address
	 */
	public $shipState;
	/**
	 * subscription/shipTo/zip (optional)
	 * The ZIP code of the customer's shipping address
	 */
	public $shipZip;
	/**
	 * subscription/shipTo/country (optional)
	 * The country of the customer's shipping address
	 */
	public $shipCountry;
	/**
	 * subscriptionId
	 * The payment gateway assigned identification number
	 * for the subscription
	 */
	public $subscriptionId;
	public $params;
	public $ddYears;
	public $ddMonths = array(
		'01'=>'Jan',
		'02'=>'Feb',
		'03'=>'Mar',
		'04'=>'Apr',
		'05'=>'May',
		'06'=>'Jun',
		'07'=>'Jul',
		'08'=>'Aug',
		'09'=>'Sep',
		'10'=>'Oct',
		'11'=>'Nov',
		'12'=>'Dec');


	public function init() {
		// Make dropdown list for years
		for ($i = date("Y"); $i<=date("Y")+10; $i++)
			$this->ddYears[$i] = $i;

	}


	/**
	 * @todo Declares the validation rules.
	 */
	public function rules() {
		return array(
			array('refId', 'length', 'max'=>20),
			array('subscriptionId', 'length', 'max'=>13),
			array('subscriptionId', 'required', 'on'=>'update, status, cancel'),
			// subscription attributes
			array('name', 'length', 'max'=>50),
			// subscription:paymentSchedule:interval
			array('length', 'valLength'),
			array('unit', 'valUnit'),
			array('startDate', 'safe'),
			array('totalOccurrences', 'safe'), // range 0 to 9999
			array('trialOccurrences', 'safe'), // range 0 to 99
			// subscription:paymentSchedule
			array('amount', 'required', 'on'=>'create'), //numeric
			array('amount', 'required', 'on'=>'create'),
			array('trialAmount', 'safe'), //Required when trial occurrences is specified.
			// subscription:payment:CreditCard
			array('cardNumber', 'required', 'on'=>'create'),
			array('cardNumber', 'valCC'),
			array('expirationDate', 'safe'), // YYYY-MM
			array('cardCode', 'safe'), // 3 to 4 digit
			// subscription:bankAccount
			array('accountType', 'safe'), // checking, businessChecking, savings
			array('routingNumber', 'safe'), // 9 digits
			array('accountNumber', 'safe'), // 5 to 17 digits
			array('nameOnAccount', 'safe'), // 22 chars
			array('echeckType', 'safe'), // checking/savings accounts={PPD,WEB}. business checking=CCD
			array('bankName', 'safe'), // 50 char
			// order
			array('invoiceNumber', 'safe'), // 20 char
			array('description', 'safe'), // 255 char
			// customer
			array('id', 'safe'), // 20 char
			array('email', 'eamil'), // 255 char
			array('phoneNumber', 'safe'), // 25 char
			array('faxNumber', 'safe'), // 25 char
			// billTo
			array('billFirstName', 'safe'), // 50 char
			array('billLastName', 'safe'), // 50 char
			array('billCompany', 'safe'), // 50 char
			array('billAddress', 'safe'), // 60 char
			array('billCity', 'safe'), // 40 char
			array('billState', 'safe'), // 2 char
			array('billZip', 'safe'), // 20 char
			array('billCountry', 'safe'), // 60 char
			// shipTo
			array('shipFirstName', 'safe'), // 50 char
			array('shipLastName', 'safe'), // 50 char
			array('shipCompany', 'safe'), // 50 char
			array('shipAddress', 'safe'), // 60 char
			array('shipCity', 'safe'), // 40 char
			array('shipState', 'safe'), // 2 char
			array('shipZip', 'safe'), // 20 char
			array('shipCountry', 'safe'), // 60 char
			// part attribes
			array('expirationYear, expirationMonth', 'safe'),
		);

	}


	public function valUnit() {
		switch (true) {
			case ($this->unit=="months"):
			case ($this->unit=="days"):
				break;
			default:
				$this->addError('length', 'Unknown unit');
				break;
		}

	}


	public function valLength() {
		switch (true) {
			case ($this->unit=="months"):
				if (($this->length<1)||($this->length>12))
					$this->addError('length', 'Range 1 to 12');
				break;
			case ($this->unit=="days"):
				if (($this->length<7)||($this->length>365))
					$this->addError('length', 'Range 7 to 365');
				break;
			default:
				$this->addError('length', 'Unknown length');
				break;
		}

	}


	/**
	 * Declares customized attribute labels.
	 * If not declared here, an attribute would have a label that is
	 * the same as its name with the first letter in upper case.
	 */
	public function attributeLabels() {
		return array(
			'billingFirstName'=>'First Name',
			'billingLastName'=>'Last Name',
			'billingCompany'=>'Company',
			'billingAddress'=>'Address',
			'billingCity'=>'City',
			'billingState'=>'State',
			'billingZip'=>'Zip',
		);

	}


	public function getParams() {
		switch ($this->scenario) {
			case 'create':
				$params = $this->getParamsCreate();
		}
		return $params;

	}


	public function getParamsCreate() {
		return array(
			'merchantAuthentication'=>array(
				'name'=>'filled by setAPI',
				'transactionKey'=>'filled by setAPI'
			),
			'refId'=>$this->refId,
			'subscription'=>array(
				'name'=>$this->name, // business name
				'paymentSchedule'=>array(
					'interval'=>array(
						'length'=>$this->length,
						'unit'=>$this->unit
					),
					'startDate'=>$this->startDate,
					'totalOccurrences'=>$this->totalOccurrences,
					'trialOccurrences'=>$this->trialOccurrences,
				),
				'amount'=>$this->amount,
				'trialAmount'=>$this->trialAmount,
				'payment'=>array(
					'creditCard'=>array(
						'cardNumber'=>$this->cardNumber,
						'expirationDate'=>$this->expirationDate,
						'cardCode'=>$this->cardCode,
					),
				),
				'bankAccount'=>array(
					'accountType'=>$this->accountType,
					'routingNumber'=>$this->routingNumber,
					'nameOnAccount'=>$this->nameOnAccount,
					'echeckType'=>$this->echeckType,
					'bankName'=>$this->bankName,
				),
				'order'=>array(
					'invoiceNumber'=>$this->invoiceNumber,
					'description'=>$this->description,
				),
				'customer'=>array(
					'id'=>$this->id,
					'email'=>$this->email,
					'phoneNumber'=>$this->phoneNumber,
					'faxNumber'=>$this->faxNumber,
				),
				'billTo'=>array(
					'company'=>$this->name,
					'firstName'=>$this->billFirstName,
					'lastName'=>$this->billLastName,
					'company'=>$this->billCompany,
					'address'=>$this->billAddress,
					'city'=>$this->billCity,
					'state'=>$this->billState,
					'zip'=>$this->billZip,
					'country'=>$this->billCountry,
				),
				'shipTo'=>array(
					'company'=>$this->name,
					'firstName'=>$this->shipFirstName,
					'lastName'=>$this->shipLastName,
					'company'=>$this->shipCompany,
					'address'=>$this->shipAddress,
					'city'=>$this->shipCity,
					'state'=>$this->shipState,
					'zip'=>$this->shipZip,
					'country'=>$this->shipCountry,
				),
			),
		);

	}


	public function getParamsCancel() {
		return array(
			'merchantAuthentication'=>array(
				'name'=>'filled by setAPI',
				'transactionKey'=>'filled by setAPI'
			),
			'refId'=>$this->refId,
			'subscriptionId'=>$this->subscriptionId,
		);

	}


	public function getParamsStatus() {
		return array(
			'merchantAuthentication'=>array(
				'name'=>'filled by setAPI',
				'transactionKey'=>'filled by setAPI'
			),
			'refId'=>$this->refId,
			'subscriptionId'=>$this->subscriptionId,
		);

	}


	public function getParamsUpdate() {
		return array(
			'merchantAuthentication'=>array(
				'name'=>'filled by setAPI',
				'transactionKey'=>'filled by setAPI'
			),
			'refId'=>$this->refId,
			'subscriptionId'=>$this->subscriptionId,
			'subscription'=>array(
				'name'=>$this->name, // business name
				'paymentSchedule'=>array(
					'interval'=>array(
						'length'=>$this->length,
						'unit'=>$this->unit
					),
					'startDate'=>$this->startDate,
					'totalOccurrences'=>$this->totalOccurrences,
					'trialOccurrences'=>$this->trialOccurrences,
				),
				'amount'=>$this->amount,
				'trialAmount'=>$this->trialAmount,
				'payment'=>array(
					'creditCard'=>array(
						'cardNumber'=>$this->cardNumber,
						'expirationDate'=>$this->expirationDate,
						'cardCode'=>$this->cardCode,
					),
				),
				'bankAccount'=>array(
					'accountType'=>$this->accountType,
					'routingNumber'=>$this->routingNumber,
					'nameOnAccount'=>$this->nameOnAccount,
					'echeckType'=>$this->echeckType,
					'bankName'=>$this->bankName,
				),
				'order'=>array(
					'invoiceNumber'=>$this->invoiceNumber,
					'description'=>$this->description,
				),
				'customer'=>array(
					'id'=>$this->id,
					'email'=>$this->email,
					'phoneNumber'=>$this->phoneNumber,
					'faxNumber'=>$this->faxNumber,
				),
				'billTo'=>array(
					'company'=>$this->name,
					'firstName'=>$this->billFirstName,
					'lastName'=>$this->billLastName,
					'company'=>$this->billCompany,
					'address'=>$this->billAddress,
					'city'=>$this->billCity,
					'state'=>$this->billState,
					'zip'=>$this->billZip,
					'country'=>$this->billCountry,
				),
				'shipTo'=>array(
					'company'=>$this->name,
					'firstName'=>$this->shipFirstName,
					'lastName'=>$this->shipLastName,
					'company'=>$this->shipCompany,
					'address'=>$this->shipAddress,
					'city'=>$this->shipCity,
					'state'=>$this->shipState,
					'zip'=>$this->shipZip,
					'country'=>$this->shipCountry,
				),
			),
		);

	}


}