<?php
class VendorController extends BaseCFCart {

	public $cartTotal=9.99;
	public $trialOccurrences = 1; //1 month Free
	public $trialAmount = 0; //1 month Free

	/**
	 * Process ARB order tansaction
	 */
	public function actionPay_order() {
		$params = $response = $result = false;

		$authf = new AuthFormARB;
		$authf->scenario = 'create';
		$authf->amount = $this->cartTotal;

		$this->performAjaxValidation($authf);

		if (isset($_POST['AuthFormARB'])) {
			$authf->attributes = $_POST['AuthFormARB'];
			$authf = $this->setARBDefault($authf); // Assign the defaults as per app

			if ($authf->validate()) {
				// Call the ARB API
				$response = $this->widget('application.extensions.authNet.authNetARB', array(
						'params'=>$authf->getParams(),
						'operation'=>$authf->scenario
					));

				$result = $response->result;

				switch ($result['status']) {
					case 'success':
						$this->successProcessOrder($authf, $result);
						break;
					case 'custom':
						$this->customErrorNoAPIResponse($authf, $result);
						break;
					case 'warn':
						$authf->addError('cardNumber', $result['text']);
						break;
					case 'error':
						$this->errorOtherExit($authf, $result);
						break;
				}
			}
		}

		$this->render('pay_order', array(
			'model'=>$this->getCFModel(),
			'authf'=>$authf,
			'result'=>$result
		));
	}


	/**
	 * Errors to abort order processing
	 * @param <type> $result
	 * @todo send notice to Admin
	 */
	public function errorOtherExit($authf, $result) {
		$this->p($result);
		die;
	}


	/**
	 * This is custom error set in the ARB API, example: No response from API
	 * @param <type> $result
	 * @todo send notice to Admin. Use fsock if curl is not an agent
	 */
	public function customErrorNoAPIResponse($authf, $result) {
		$order = new CFOrderLog;
		$order->created = date("Y-m-d h:i:s");
		$order->attributes = $authf->attributes;
		$order->cardNumber = preg_replace('/(?!^.?)[0-9](?!(.){0,3}$)/', '*', '3456-7890-1234-5678');
		$order->attributes = $result;
		$order->is_valid = 0;
		$order->save();
		$this->redirect(array('response_custom_error', 'id'=>$order->id));
	}


	public function actionResponse_custom_error($id) {
		$this->render('response_custom_error', array(
			'order'=>CFOrderLog::model()->findByPk($id)
		));
	}


	/**
	 * Success Oreder Processing
	 * @param <type> $result
	 * @todo send notice with uid and password, create ldb if needed
	 */
	public function successProcessOrder($authf, $result) {
		// on payment success
		$order = new CFOrderLog;
		$order->created = date("Y-m-d h:i:s");
		$order->attributes = $authf->attributes;
		// mask CC
		$order->cardNumber = preg_replace('/(?!^.?)[0-9](?!(.){0,3}$)/', '*', '3456-7890-1234-5678');
		$order->attributes = $result;
		$order->is_valid = 1;
		$order->save();
		$this->createCatalog();
		$this->redirect(array('thank_you'));
	}


	/**
	 * Set the default to the CFrom as per this subscription
	 * @param <type> $authf
	 * @return <type>
	 */
	public function setARBDefault($authf) {
		$nxt_order_id = CFOrder::model()->getNextId();
		$authf->refId = 'R'.$nxt_order_id;
		$authf->startDate = date("Y-m-d");
		$authf->unit = 'months';
		$authf->length = 1; // monthly
		$authf->expirationDate = $authf->expirationYear.'-'.$authf->expirationMonth;
		$authf->trialOccurrences = $this->trialOccurrences;
		$authf->trialAmount = $this->trialAmount;
		$authf->invoiceNumber = date("Ym").'-'.$nxt_order_id;
		$authf->description = 'Catalog insertion monthly fee.';
		return $authf;
	}


	/**
	 * Order recieved thankyou page
	 */
	public function actionThank_you() {
		// Conditional rMenu for layout Catalog
		$this->rMenu = array(
			array('label'=>'Add my catalog', 'url'=>'/gfied/catalogs/vendor/add_catalog/'),
			array('label'=>'Broswe Catalog', 'url'=>'/gfied/catalogs/default/')
		);
		$this->render('thank_you');
	}

	/**
	 * Performs the AJAX validation.
	 * @param CModel the model to be validated
	 */
	protected function performAjaxValidation($model) {
		if (isset($_POST['ajax'])&&$_POST['ajax']==='authorize-form') {
			echo CActiveForm::validate($model);
			Yii::app()->end();
		}
	}


}